<!DOCTYPE html
    PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"
      xmlns:xi="http://www.w3.org/2001/XInclude"
      xmlns:py="http://genshi.edgewall.org/"
      xmlns:i18n="http://genshi.edgewall.org/i18n" i18n:domain="tracspamfilter">
  <xi:include href="admin.html" />
  <head>
    <title>Spam Report</title>
  </head>

  <body>
    <div class="nav">
      <ul py:with="links = chrome.links"
          py:if="'up' in chrome.links or 'prev' in chrome.links or 'next' in chrome.links">
        <li class="first" py:choose="">
          &larr; <a py:when="'prev' in links" py:with="link = links.prev[0]"
                    class="prev" href="${link.href}"
                    title="${link.title}">Previous Report Entry</a>
          <span py:otherwise="" class="missing">Previous Report Entry</span>
        </li>
        <li py:if="'up' in links">
          <a py:with="link = links.up[0]" href="${link.href}"
             title="${link.title}">Back to List</a>
        </li>
        <li class="last" py:choose="">
          <a py:when="'next' in links"  py:with="link = links.next[0]"
             class="next" href="${link.href}"
             title="${link.title}">Next Report Entry</a>
          <span py:otherwise="" class="missing">Next Report Entry</span> &rarr;
        </li>
      </ul>
    </div>
    <h2>Spam Filtering: Report</h2>

    <form class="mod" id="reportentry" action="" method="post">
      <fieldset>
        <legend>Report Entry:</legend>
        <h3>Information</h3>
        <table class="meta"><tr>
          <th>Time:</th>
          <td>${pretty_dateinfo(fromtimestamp(time))}</td>
        </tr><tr>
          <th>Path:</th>
          <td><a href="${href(path)}">${abs_href(path)}</a></td>
        </tr><tr>
          <th>Author:</th>
          <td>${author}</td>
        </tr><tr>
          <th>Authenticated:</th>
          <td>${authenticated and dgettext('tracspamfilter', 'yes') or dgettext('tracspamfilter', 'no')}</td>
        </tr><tr>
          <th>Comment:</th>
          <td>${comment}</td>
        </tr></table>
        <div class="headers">
          <h3>HTTP headers</h3>
          <pre>${headers}</pre>
        </div>
        <div class="buttons">
          <input type="hidden" name="sel" value="${id}" />
          <input type="submit" name="delete" value="${dgettext('tracspamfilter', 'Delete')}" />
        </div>
      </fieldset>
    </form>

    <h3>Possibly related log entries:</h3>

    <xi:include href="monitortable.html" />
  </body>
</html>
