<!DOCTYPE html
    PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"
      xmlns:xi="http://www.w3.org/2001/XInclude"
      xmlns:py="http://genshi.edgewall.org/"
      xmlns:i18n="http://genshi.edgewall.org/i18n" i18n:domain="tracspamfilter">
  <xi:include href="admin.html" />
  <head>
    <title>Spam Monitoring</title>
  </head>

  <body>
    <h2>Spam Filtering: Monitoring</h2>

    <form method="post" action="" id="spammonitorform">
      <p py:if="not enabled">
        <strong>Note:</strong> Logging by the spam filter is currently disabled.
      </p>
      <p class="hint" py:if="total" i18n:msg="start, end, total">
        Viewing entries ${offset} – ${offset + len(entries) - 1} of ${total}.
      </p>

      <ul py:def="prevnext_nav()" py:with="links = chrome.links"
           py:if="'prev' in chrome.links or 'next' in chrome.links">
        <li class="first" py:choose="">
          &larr; <a py:when="'prev' in links" py:with="link = links.prev[0]"
                    class="prev" href="${link.href}"
                    title="${link.title}">Previous Page</a>
          <span py:otherwise="" class="missing">Previous Page</span>
        </li>
        <li class="last" py:choose="">
          <a py:when="'next' in links"  py:with="link = links.next[0]"
             class="next" href="${link.href}"
             title="${link.title}">Next Page</a>
          <span py:otherwise="" class="missing">Next Page</span> &rarr;
        </li>
      </ul>

      <div class="nav" py:if="entries">
        ${prevnext_nav()}
      </div>

      <xi:include href="monitortable.html" />

      <div class="nav" py:if="entries">
        ${prevnext_nav()}
      </div>
      <div id="boxes" py:if="entries"></div>

      <div class="buttons" py:if="entries">
        <input type="submit" class="trac-disable-on-submit"
               name="deleteobvious" value="${dgettext('tracspamfilter', 'Delete &gt; 90%')}" />
        <input type="hidden" name="page" value="$page" />
        <input type="hidden" name="num" value="$num" />
        <input py:if="train_only" class="spambutton trac-disable-on-submit"
               type="submit" name="markspam" value="${dgettext('tracspamfilter', 'Mark selected as Spam')}" />
        <input py:if="train_only" class="hambutton trac-disable-on-submit"
               type="submit" name="markham" value="${dgettext('tracspamfilter', 'Mark selected as Ham')}" />
        <input type="submit" class="trac-disable-on-submit"
               name="delete" value="${dgettext('tracspamfilter', 'Delete selected')}" />
        <input type="submit"  class="spambutton trac-disable-on-submit"
               name="markspamdel" value="${dgettext('tracspamfilter', 'Delete selected as Spam')}" />
        <input type="submit" class="hambutton trac-disable-on-submit"
               name="markhamdel" value="${dgettext('tracspamfilter', 'Delete selected as Ham')}" />
      </div>
    </form>
  </body>

</html>
